'use strict';

let load = (() => {
  var _ref = _asyncToGenerator(function* () {
    let dynamicDefinition = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    let globals = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    globals.raw = {};
    // load model, service and middlewares
    ['model', 'service', 'middleware'].forEach(function (type) {
      const name = type + 's';
      globals[name] = {};
      try {
        const dir = joinPath(rc.target, type);
        fs.statSync(dir);
        const modules = moder(dir, {
          lazy: false,
          filter: isTest
        });
        debug(type, Object.keys(modules));
        globals[name] = modules;
      } catch (err) {
        if (err.code !== 'ENOENT') {
          throw err;
        }
      }
    });

    // wrap normal middlewares
    globals.raw.middlewares = {};
    _.forEach(globals.middlewares, function (middleware, name) {
      globals.raw.middlewares[name] = middleware;
      globals.middlewares[name] = function () {
        return wrapMiddleware(middleware.apply(undefined, arguments));
      };
    });
    // alias
    globals.mws = globals.middlewares;

    // load swagger definitions
    globals.api = yield Swagger.createAsync(dynamicDefinition);

    // read api implementation and bind them into definition
    const pathDefs = globals.api.definition.paths;
    glob.sync(`**/*${rc.suffix.code}`, { cwd: rc.target }).sort().forEach(function (file) {
      const codePath = joinPath(rc.target, file);
      debug('require', codePath);
      const mod = require(codePath);
      if (rc.shadow) {
        _.forEach(mod, function (handlers, path) {
          bindHandlers(pathDefs, handlers, path);
        });
      } else {
        const path = fromCode(file);
        bindHandlers(pathDefs, mod, path);
      }
    });

    return globals;
  });

  return function load() {
    return _ref.apply(this, arguments);
  };
})();

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

/**
 * Load middleware, service and api implementations
 */
const fs = require('fs');
const joinPath = require('path').join;

const debug = require('debug')('ruo');
const moder = require('moder');
const _ = require('lodash');
const glob = require('glob');

const rc = require('./rc');

var _require = require('./utility');

const fromCode = _require.fromCode,
      isTest = _require.isTest,
      wrapMiddleware = _require.wrapMiddleware;

var _require2 = require('./swagger');

const Swagger = _require2.Swagger;


function bindHandlers(pathDefs, handlers, path) {
  _.forEach(handlers, (handler, method) => {
    // bind handler only when api is defined
    const operationDef = _.get(pathDefs, [path, method]);
    if (operationDef) {
      operationDef.__handler__ = handler;
    }
  });
}

module.exports = load;